#!/usr/bin/python
# -*- coding: utf-8 -*-

# Screen
from Screens.Screen import Screen
from Screens.MessageBox import MessageBox
from Screens.HelpMenu import HelpableScreen

# GUI (Summary)
from Screens.Setup import SetupSummary

from Components.Label import Label, BlinkingLabel
from Components.ScrollLabel import ScrollLabel
from Components.Sources.StaticText import StaticText
from Components.ActionMap import ActionMap, HelpableActionMap
from Components.Pixmap import Pixmap
from Plugins.Plugin import PluginDescriptor
from Components.PluginComponent import plugins

# GUI (System) 
from enigma import eTimer, ePoint, eSize, getDesktop, ePoint, eConsoleAppContainer

# Configuration
from Components.config import *
from Components.ConfigList import ConfigListScreen

from Tools.Notifications import AddPopup
from Tools.Directories import fileExists, resolveFilename, SCOPE_PLUGINS
from skin import parseColor

import Screens.Standby
import os, sys, traceback
import subprocess, threading
import time
from datetime import datetime, timedelta

PluginVersion = ""
reload_value = True

sz_w = getDesktop(0).size().width()

class UpdateCheckSetup(Screen, ConfigListScreen):
	
	if sz_w == 1920:
		skin = """
		<screen name="UpdateCheckSetup" position="center,center" size="1200,820">
			<widget name="logo" position="10,10" size="150,60" />
			<ePixmap pixmap="skin_default/buttons/red.png" position="170,5" size="255,70" scale="stretch" alphatest="on" />
			<ePixmap pixmap="skin_default/buttons/green.png" position="425,5" size="255,70" scale="stretch" alphatest="on" />
			<ePixmap pixmap="skin_default/buttons/yellow.png" position="680,5" size="255,70" scale="stretch" alphatest="on" />
			<ePixmap pixmap="skin_default/buttons/blue.png" position="935,5" size="255,70" scale="stretch" alphatest="on" />
			<ePixmap pixmap="skin_default/div-h.png" position="10,80" zPosition="2" size="1180,2" />
			<widget source="key_red" render="Label" position="170,5" zPosition="1" size="255,70" font="Regular;30" halign="center" valign="center" backgroundColor="#9f1313" shadowColor="black" shadowOffset="-2,-2" transparent="1" />
			<widget source="key_green" render="Label" position="425,5" zPosition="1" size="255,70" font="Regular;30" halign="center" valign="center" backgroundColor="#1f771f" shadowColor="black" shadowOffset="-2,-2" transparent="1" />
			<widget source="key_yellow" render="Label" position="680,5" zPosition="1" size="255,70" font="Regular;30" halign="center" valign="center" backgroundColor="#a08500" shadowColor="black" shadowOffset="-2,-2" transparent="1" />
			<widget source="key_blue" render="Label" position="935,5" zPosition="1" size="255,70" font="Regular;30" halign="center" valign="center" backgroundColor="#18188b" shadowColor="black" shadowOffset="-2,-2" transparent="1" />
			<widget name="config" position="10,90" size="1180,540" enableWrapAround="1" scrollbarMode="showOnDemand" />
			<ePixmap pixmap="skin_default/div-h.png" position="10,650" zPosition="2" size="1180,2" />
			<widget name="help" position="10,655" size="1180,145" font="Regular;32" />
		</screen>"""
	else:
		skin = """
		<screen name="UpdateCheckSetup" position="center,center" size="800,530">
			<widget name="logo" position="5,2" size="100,40" />
			<ePixmap pixmap="skin_default/buttons/red.png" position="110,0" size="170,45" scale="stretch" alphatest="on" />
			<ePixmap pixmap="skin_default/buttons/green.png" position="280,0" size="170,45" scale="stretch" alphatest="on" />
			<ePixmap pixmap="skin_default/buttons/yellow.png" position="450,0" size="170,45" scale="stretch" alphatest="on" />
			<ePixmap pixmap="skin_default/buttons/blue.png" position="620,0" size="170,45" scale="stretch" alphatest="on" />
			<ePixmap pixmap="skin_default/div-h.png" position="5,50" zPosition="2" size="790,2" />
			<widget source="key_red" render="Label" position="110,0" zPosition="1" size="170,45" font="Regular;22" halign="center" valign="center" backgroundColor="#9f1313" shadowColor="black" shadowOffset="-2,-2" transparent="1" />
			<widget source="key_green" render="Label" position="280,0" zPosition="1" size="170,45" font="Regular;22" halign="center" valign="center" backgroundColor="#1f771f" shadowColor="black" shadowOffset="-2,-2" transparent="1" />
			<widget source="key_yellow" render="Label" position="450,0" zPosition="1" size="170,45" font="Regular;22" halign="center" valign="center" backgroundColor="#a08500" shadowColor="black" shadowOffset="-2,-2" transparent="1" />
			<widget source="key_blue" render="Label" position="620,0" zPosition="1" size="170,45" font="Regular;22" halign="center" valign="center" backgroundColor="#18188b" shadowColor="black" shadowOffset="-2,-2" transparent="1" />
			<widget name="config" position="5,60" size="790,380" enableWrapAround="1" scrollbarMode="showOnDemand" />
			<ePixmap pixmap="skin_default/div-h.png" position="5,431" zPosition="2" size="790,2" />
			<widget name="help" position="5,440" size="790,75" font="Regular;20" />
		</screen>"""


	def __init__(self, session):
		Screen.__init__(self, session)

		# Summary
		global PluginVersion
		from plugin import VERSION
		PluginVersion = VERSION
		self.skinName = "UpdateCheckSetup"
		self.setup_title = _("UpdateCheck Setup ") + str(PluginVersion)
		self.onChangedEntry = []
		
		self.list = []
		self.buildConfig()
		ConfigListScreen.__init__(self, self.list, session = session, on_change = self.changed)
		#if sz_w == 1920:
		#	self["config"].l.setSeperation(900)
		#else:
		#	self["config"].l.setSeperation(600)
	
		def selectionChanged():
			if self["config"].current:
				self["config"].current[1].onDeselect(self.session)
			self["config"].current = self["config"].getCurrent()
			if self["config"].current:
				self["config"].current[1].onSelect(self.session)
			for x in self["config"].onSelectionChanged:
				x()
		self["config"].selectionChanged = selectionChanged
		self["config"].onSelectionChanged.append(self.updateHelp)

		# Initialize widgets
		self["logo"] = Pixmap()
		self["key_green"] = StaticText(_("OK"))
		self["key_red"] = StaticText(_("Cancel"))
		self["key_yellow"] = StaticText("Update-Info")
		self["key_blue"] = StaticText("Update-Check")
		self["help"] = Label("")
		
		# Define Actions
		self["actions"] = ActionMap(["SetupActions", "ColorActions"],
			{
				"cancel": 	self.keyCancel,
				"save": 		self.keySave,
				"yellow":		self.updateInfo,
				"blue":			self.test,
				"ok": 			self.ok,
			},-2)

		# Trigger change
		self.changed()

		self.onLayoutFinish.append(self.layoutFinished)

	def buildConfig(self):
		
		self.list.append( getConfigListEntry(_("Updatecheck und Info beim Start der Box"), config.plugins.updatecheck.enable_autocheck, _("Prüft beim Box-Start (auch GUI-Neustart und Start aus dem Idle-Mode) auf Updates und gibt eine Meldung aus (nein/nur bei Updates/immer). Bei 'immer' kommt auch eine Info-Meldung, dass kein Update verfügbar ist.")) )
		
		if config.plugins.updatecheck.enable_autocheck.value != "False":
			self.list.append( getConfigListEntry(_("   Intervall für AutoUpdateCheck beim Start der Box"), config.plugins.updatecheck.autoCheck_interval, _("Legt den Prüf-Intervall für den AutoUpdateCheck beim Start der Box fest. Innerhalb des Intervalls erfolgt kein erneuter Update-Check beim Start der Box.")) )
			
			self.list.append( getConfigListEntry(_("   Update-Verzögerung nach Box-Start (in Sek)"), config.plugins.updatecheck.updatecheck_delay, _("Legt fest, mit wieviel Sekunden Verzögerung der Updatecheck nach Boxstart ausgeführt wird.")) )
			
			self.list.append( getConfigListEntry(_("   UpdateCheck-Timeout (in Sek)"), config.plugins.updatecheck.update_timeout, _("Legt fest, nach wieviel Sekunden die Update-Prozesse z.B. bei Update-Server-Problemen spätestens beendet werden sollen (5-30, default=5)")) )
			
			self.list.append( getConfigListEntry(_("   Anzeigedauer für Update-Info-Fenster (in Sek)"), config.plugins.updatecheck.popups_timeout, _("Legt fest, nach wieviel Sekunden das Update-Info-Fenster automatisch geschlossen wird (1-20 Sek., 0 = wartet auf Tastendruck)")) )
			
			self.list.append( getConfigListEntry(_("   Ingnoriere hold-Pakete bei der Update-Info"), config.plugins.updatecheck.ignore_hold, _("Pakete, welche auf 'hold' gesetzt wurden, werden bei 'Ja' bei der Update-Prüfung nicht gemeldet.")) )
			
			self.list.append( getConfigListEntry(_("   Update-Info mit Frage zum Öffnen des SoftwareManagers"), config.plugins.updatecheck.show_updatequestion, _("Zeigt beim Update-Hinweis zusätzlich eine Frage, ob man den SoftwareManager direkt öffnen möchte.")) )
			
			if config.plugins.updatecheck.show_updatequestion.value is True:
				self.list.append( getConfigListEntry(_("    ... Antwort für Frage steht standardmäßig auf:"), config.plugins.updatecheck.updatequestion_defaultanswer, _("Legt fest, welche Antwort für die Frage zum Öffnen des SoftwareManagers standardmäßig vorausgewählt sein soll. Bei 'nein' öffnet sich der SoftwareManager bei Zeitablauf der Frage nicht automatisch.")) )
			
			self.list.append( getConfigListEntry(_("Zeige Befehl für den UpdateCheck im Erweiterungsmenü"), config.plugins.updatecheck.show_extmenu, _("Zeigt einen Eintrag im Erweiterungsmenü, um direkt einen Updatecheck auszuführen.")) )
			self.list.append( getConfigListEntry(_("Zeige Befehl für das UpdateCheck-Setup"), config.plugins.updatecheck.show_setup, _("Zeigt einen Eintrag zum Öffnen des Updatecheck-Setup an (im PluginBrowser, im Erweiterungsmenü oder an beiden Stellen).")) )

	def layoutFinished(self):
		self["logo"].instance.setPixmapFromFile("/usr/lib/enigma2/python/Plugins/Extensions/UpdateCheck/UpdateCheck.png")
		self.setTitle(self.setup_title)

	def ok(self):
		pass
		#if self["config"].getCurrent()[1] == config.plugins.updatecheck. ...:
		#	pass

	def updateHelp(self):
		cur = self["config"].getCurrent()
		if cur:
			self["help"].text = cur[2]

	def changeConfig(self):
		self.list = []
		self.buildConfig()
		self["config"].setList(self.list)

	def change_MenuEntry(self, entry="extMenuCheck"):
		if entry == "extMenuCheck":
			from plugin import extMenuDescriptor as pd
			change = config.plugins.updatecheck.show_extmenu.value
		if entry == "pbMenuSetup":
			from plugin import pbrowserSetupDescriptor as pd
			change = config.plugins.updatecheck.show_setup.value in ("pbrowser","both")
		if entry == "extMenuSetup":
			from plugin import extMenuSetupDescriptor as pd
			change = config.plugins.updatecheck.show_setup.value in ("extmenu","both")
		#print "=== menu", pd.name, pd.where, change
		#print "=== pd", pd.__dict__
		pdList = [(p) for p in plugins.getPlugins(where = pd.where[:]) if p is pd]
		if change:
			if pd not in pdList:
				if pd.icon is None:
					pd.updateIcon("/usr/lib/enigma2/python/Plugins/Extensions/UpdateCheck")
				plugins.addPlugin(pd)
		else:
			for pd in pdList:
				plugins.removePlugin(pd)

	def keySave(self):
		self.saveAll()
		for entry in ("extMenuCheck","pbMenuSetup","extMenuSetup"):
			self.change_MenuEntry(entry)
		self.close()

	def changed(self):
		for x in self.onChangedEntry:
			x()
		current = self["config"].getCurrent()[1]
		if (current == config.plugins.updatecheck.enable_autocheck) or (current == config.plugins.updatecheck.show_updatequestion):
			self.changeConfig()
			return

	def getCurrentEntry(self):
		return self["config"].getCurrent()[0]

	def getCurrentValue(self):
		return str(self["config"].getCurrent()[1].getText())

	def createSummary(self):
		return SetupSummary

	def test(self):
		self.session.open(UpdateScreen)

	def updateInfo(self):
		try:
			check_interval = [x for x in config.plugins.updatecheck.autoCheck_interval.choices.choices if config.plugins.updatecheck.autoCheck_interval.value == x[0]][0][1]
			lastcheck = datetime.fromtimestamp(int(config.plugins.updatecheck.lastAutoCheck.value))
			text = ('UpdateCheck Update-Info\n\nLetzter Update-Check: \t%s' % lastcheck).expandtabs(14)
			if config.plugins.updatecheck.enable_autocheck.value:
				#calculate the time for next updateCheck
				lastcheck = int(config.plugins.updatecheck.lastAutoCheck.value)
				if config.plugins.updatecheck.autoCheck_interval.value == "1":
					nextcheck = datetime.fromtimestamp(lastcheck) + timedelta(hours=1)
				elif config.plugins.updatecheck.autoCheck_interval.value == "2":
					nextcheck = datetime.fromtimestamp(lastcheck) + timedelta(hours=6)
				elif config.plugins.updatecheck.autoCheck_interval.value == "3":
					nextcheck = datetime.fromtimestamp(lastcheck) + timedelta(hours=12)
				elif config.plugins.updatecheck.autoCheck_interval.value == "4":
					nextcheck = datetime.fromtimestamp(lastcheck) + timedelta(days=1)
				elif config.plugins.updatecheck.autoCheck_interval.value == "5":
					nextcheck = datetime.fromtimestamp(lastcheck) + timedelta(weeks=1)
				elif config.plugins.updatecheck.autoCheck_interval.value == "6":
					# goto next month
					next_month = datetime.fromtimestamp(lastcheck).replace(day=28) + timedelta(days=4)
					# reset day in the next month date
					nextcheck = next_month.replace(day=datetime.fromtimestamp(lastcheck).day) 
				if config.plugins.updatecheck.autoCheck_interval.value != "0":
					text += ('\nnächster Update-Check: \t%s' % nextcheck).expandtabs(3)
				text += ('\nCheck-Interval: \t%s' % check_interval).expandtabs(34)
			self.session.open(MessageBox, text, MessageBox.TYPE_INFO)
		except:
			import traceback
			traceback.print_exc()

########################################################################### 

class Command(object):
	def __init__(self, cmd):
		self.cmd = cmd
		self.process = None

	def run(self, timeout):
		def target():
			#print "[UpdateCheck] command-Thread started"
			self.process = subprocess.Popen(self.cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
			(self.out,self.err) = self.process.communicate()
			#print "[UpdateCheck] command-Thread finished"

		thread = threading.Thread(target=target)
		thread.start()
		self.timeout=False

		thread.join(float(timeout))
		if thread.is_alive():
			print "[UpdateCheck] Terminating command-process by timeout:", self.cmd
			self.process.terminate()
			#print '[UpdateCheck] timeout=True'
			self.timeout=True
			self.process = None
			thread = None
			return
			#print '[UpdateCheck] before thread.join()'
			#thread.join()
			#print '[UpdateCheck] after thread.join()'

class UpdateScreen(Screen, HelpableScreen):
	
	if sz_w == 1920:
		skin = """
		<screen name="UpdateCheck" position="center,center" size="1200,680">
			<widget name="myLabel" position="15,20" size="650,80" font="Regular;24"/>
			<widget name="status" position="15,120" size="650,110" font="Regular;32"/>
			<widget name="updates" position="15,230" size="1150,400" font="Regular;20"/>
			<widget source="global.CurrentTime" render="Label" position="700,15" size="475,40" font="Regular;32" halign="right">
				<convert type="ClockToText">Format:%A %d.%m.  %H:%M</convert>
			</widget>
			<widget name="updatestart" position="800,60" size="370,40" foregroundColor="black" backgroundColor="yellow" valign="center" halign="center" font="Regular;24"/>
		</screen>"""
	else:
		skin = """
		<screen name="UpdateCheck" position="center,center" size="930,520">
			<widget name="myLabel" position="15,20" size="500,50" font="Regular;18"/>
			<widget name="status" position="15,100" size="750,120" font="Regular;24"/>
			<widget name="updates" position="15,180" size="900,320" font="Regular;16"/>
			<widget source="global.CurrentTime" render="Label" position="540,20" size="380,30" font="Regular;24" halign="right">
				<convert type="ClockToText">Format:%A %d.%m.  %H:%M</convert>
			</widget>
			<widget name="updatestart" position="650,60" size="250,30" foregroundColor="black" backgroundColor="yellow" valign="center" halign="center" font="Regular;18"/>
		</screen>"""
	
	def __init__(self, session, args = 0):
		self.session = session
		Screen.__init__(self, session)
		HelpableScreen.__init__(self)
		
		try:
			self["myLabel"] = Label("")
			self["status"] = BlinkingLabel("") #Label("")
			self["updatestart"] = Label("Updatestart (Menü-Taste)")
			self["updates"] = ScrollLabel("")
			global PluginVersion
			from plugin import VERSION
			PluginVersion = VERSION
			self.skinName = "UpdateCheck"
			
			self.isUpdatestart = False
			self["updatestart"].hide()
			
			self["myActionMap"] = ActionMap(["InfobarEPGActions", "OkCancelActions", "MovieSelectionActions", "DirectionActions"],
				{
						"cancel"			 : self.cancel,
						"contextMenu"  	 	 : self.updatestart,
						"showEventList"		 : self.updatestart,
						"ok"				 : self.ok,
						"up":    self["updates"].pageUp,
						"down":  self["updates"].pageDown,
				},-1)
			
			# add helpstrings to helpscreen
			self.helpList.append((self["myActionMap"], "OkCancelActions", [("cancel", _("Plugin beenden"))]))
			self.helpList.append((self["myActionMap"], "OkCancelActions", [("ok", _("Updateprüfung"))]))

			self.onLayoutFinish.append( self.LayoutFinish )

		except:
			import traceback
			traceback.print_exc()

	def LayoutFinish(self):
		self.setTitle("UpdateCheck - " + PluginVersion)
		self["myLabel"].setText(_("Zum Prüfen die OK-Taste drücken."))
		self.ok()

	def updatestart(self):
		if self.isUpdatestart:
			default_answer = False
			if config.plugins.updatecheck.updatequestion_defaultanswer.value == "True":
				default_answer = True
			self.session.openWithCallback(self.openUDC, MessageBox, "\n = UpdateCheck = \n\nSoll der SoftwareManager geöffnet werden?", MessageBox.TYPE_YESNO, timeout = int(config.plugins.updatecheck.popups_timeout.value), default = default_answer)
	
	def openUDC(self, confirm):
		
		if confirm:
			#check to open GP3/GP4 AddonManager
			if fileExists(resolveFilename(SCOPE_PLUGINS, "GP4/geminiaddonmanager/gaddon.pyo")):
				print "[UpdateCheck] import GP4 gAddons"
				from Plugins.GP4.geminiaddonmanager.gaddon import gAddonScreen
				self.session.open(gAddonScreen)
				self.close()
			
			#check to open GP3 AddonManager
			elif fileExists(resolveFilename(SCOPE_PLUGINS, "Bp/geminimain/gAddons.pyo")):
				print "[UpdateCheck] import GP3 gAddons"
				from Plugins.Bp.geminimain.gAddons import gAddons
				self.session.open(gAddons)
				self.close()
			
			#check for Merlin
			elif isMerlin and fileExists(resolveFilename(SCOPE_PLUGINS, "SystemPlugins/SoftwareManager/UpdatePlugin.py")):
				self.session.openWithCallback(self.runUpgrade_Merlin, MessageBox, _("Do you want to update your Dreambox?")+"\n"+_("\nAfter pressing OK, please wait!"))
			
			#check for Newnigma2
			elif fileExists(resolveFilename(SCOPE_PLUGINS, "newnigma2/paketmanager/Paketmanager.pyo")):
				from Plugins.newnigma2.tools.NewNigma2Stuff import __
				message = __('Do you really want to Upgrade?')
				self.session.openWithCallback(self.runUpgrade_Newnigma2, MessageBox, message, type=MessageBox.TYPE_YESNO, timeout=20, default=False)
			
			#check to open DMM-SoftwareManager
			elif fileExists(resolveFilename(SCOPE_PLUGINS, "SystemPlugins/SoftwareManager/plugin.py")):
				from Plugins.SystemPlugins.SoftwareManager.plugin import PluginManager
				self.session.open(PluginManager)
				self.close()
			else:
				self.session.open(MessageBox, _("The Softwaremanagement extension is not installed!\nPlease install it."), type = MessageBox.TYPE_INFO,timeout = 10 )
				self.close()

	def runUpgrade_Merlin(self, result):
		if result:
			from Plugins.SystemPlugins.SoftwareManager.UpdatePlugin import UpdatePlugin
			self.session.open(UpdatePlugin)
		self.close()

	def runUpgrade_Newnigma2(self, result):
		if result:
			from Plugins.newnigma2.paketmanager.Paketmanager import InstallLog
			self.session.open(InstallLog, 3, None, None, "/usr/lib/enigma2/python/Plugins/newnigma2/paketmanager")
		self.close()

	def setup(self):
		self.session.open(UpdateCheckSetup)

	def ok(self):
		self["status"].instance.setForegroundColor(parseColor("foreground"))
		self["status"].setText(_("Prüfe Update-Status. Bitte warten ...\n(apt-get update)"))
		#self["status"].setBlinkTime(450)
		#self["status"].startBlinking()
		self.isUpdatestart = False
		self["updatestart"].hide()
		self["myLabel"].hide()
		self["updates"].setText("")
		#self["updates"].hide()
		self.startUpdate()

	def startUpdate(self):
		self.timeoutTimer = eTimer()
		self.timeoutTimer_conn = self.timeoutTimer.timeout.connect(self.CheckTimeout)
		self.timeoutTimer.start(int(config.plugins.updatecheck.update_timeout.value)*1000)
		
		self.container = eConsoleAppContainer()
		self.appClosed_conn = self.container.appClosed.connect(self.checkUpdate)
		self.appErr_conn = self.container.stderrAvail.connect(self.consoleError)
		self.dataAvail_conn = self.container.dataAvail.connect(self.consoledataAvail)
		self.stdoutAvail_conn = self.container.stdoutAvail.connect(self.consolestdoutAvail)
		self.container_error=False
		self.container.execute("apt-get update")
		print "container.execute(apt-get update)"

	def CheckTimeout(self):
		print "[UpdateCheck] CheckTimeout"
		
		self.timeoutTimer.stop()
		self.timeoutTimer = None
		self.timeoutTimer_conn = None
		
		if self.container.running():
			print "[UpdateCheck] timeout for eConsoleAppContainer"
			self.container.kill()
			self["status"].setText("Updateckeck timeout")
			self["status"].instance.setForegroundColor(parseColor("red"))
			self["status"].stopBlinking()
			self["status"].show()
			self["myLabel"].show()

	def consoleError(self, retval):
		print "[UpdateCheck] consoleError", retval
		self["status"].setText("Updatecheck error")
		self["status"].instance.setForegroundColor(parseColor("red"))
		self["status"].stopBlinking()
		self["status"].show()
		self["myLabel"].show()
		self["updates"].setText(retval)
		self.container_error=True

	def consolestdoutAvail(self, str):
		print "[UpdateCheck] consolestdoutAvail", str
		self["updates"].appendText(str)
		self["updates"].lastPage()

	def consoledataAvail(self, str):
		#print "== consoledataAvail", str
		pass

	def checkUpdate(self, retval):
			
		self.timeoutTimer.stop()
		self.timeoutTimer = None
		self.timeoutTimer_conn = None

		if self.container_error:
			return
		
		#print "[UpdateCheck] retval", retval
		
		command = Command("apt list --upgradable")
		command.run(timeout=int(config.plugins.updatecheck.update_timeout.value))
		
		if command.timeout:
			self["status"].setText("list --upgradable timeout")
			self["status"].instance.setForegroundColor(parseColor("red"))
			self["status"].stopBlinking()
			self["status"].show()
			self["myLabel"].show()
			return
		elif command.process.returncode != 0:
			self["status"].setText("list --upgradable error: " + str(command.process.returncode) + " - " + str(command.err))
			self["status"].instance.setForegroundColor(parseColor("red"))
			self["status"].stopBlinking()
			self["status"].show()
			self["myLabel"].show()
			return
		
		check_update_txt = command.out
		
		updates = check_update_txt.split("\n")
		all_updates = list(updates)
		#print "[UpdateCheck] check_update:", check_update_txt
		#print "[UpdateCheck] update:", updates, len(updates)
		
		hold_packages = []
		
		try:
			# print "[UpdateCheck] check hold_packages"
			hold_packages_txt = subprocess.check_output("apt-mark showhold", shell=True)

			# prepare hold-list
			if len(hold_packages_txt):
				packages = hold_packages_txt.split("\n")
				for package in packages:
					if package: hold_packages.append(str(package) + "/")
			
			#clean updates-list by hold-list if option is set to ignore hold-packages
			if config.plugins.updatecheck.ignore_hold.value:
				for hold_package in hold_packages:
					index = [i for i,x in enumerate(updates) if hold_package in x]
					if len(index):
						#print "== index:", index, updates[index[0]]
						del updates[index[0]]
			
		except:
			import traceback
			traceback.print_exc()
		
		#print "== hold_packages:", hold_packages
		
		if len(updates)==3:
			self["status"].setText("Es liegt 1 Feed-Update vor!")
			print "[UpdateCheck] 1 update found"
			self["status"].instance.setForegroundColor(parseColor("yellow"))
			self.isUpdatestart = True
		elif len(updates)>3:
			self["status"].setText("Es liegen " + str(len(updates)-2) + " Feed-Updates vor!")
			print "[UpdateCheck] %s updates found" % str(len(updates)-2)
			self["status"].instance.setForegroundColor(parseColor("yellow"))
			self.isUpdatestart = True
		else:
			if all_updates != updates:
				self["status"].setText("keine Feed-Updates gefunden!\n(nur hold-Updates)")
				self["status"].instance.setForegroundColor(parseColor("green"))
			else:
				self["status"].setText("keine Feed-Updates gefunden!")
				self["status"].instance.setForegroundColor(parseColor("green"))
			print "[UpdateCheck] no updates found"
		
		self["status"].stopBlinking()
		self["status"].show()
		self["myLabel"].show()
		
		if self.isUpdatestart:
			self["updatestart"].show()
		
		try:
			all_updates_txt = ""
			hold_updates_txt = ""
			for update in all_updates[1:]:
				if update.split("/")[0]+"/" in hold_packages:
					hold_updates_txt += "\n- " + update
				elif update:
					all_updates_txt += "\n- " + update
			
			if all_updates_txt:
				all_updates_txt = "Updates (%s):%s\n" % (len(updates)-2, all_updates_txt)
			else:
				all_updates_txt = "Updates (0):\n- keine\n"
			if hold_updates_txt:
				hold_updates_txt = "\nhold-Updates (%s):%s\n" % (len(hold_updates_txt.split('\n'))-1, hold_updates_txt)
			
			if len(hold_packages):
				self["updates"].setText(all_updates_txt + hold_updates_txt + "\nhold-Packages (%s):\n" % len(hold_packages) + "\n".join("- %s" % i for i in hold_packages))
			else:
				self["updates"].setText(all_updates_txt + hold_updates_txt + "\nhold-Packages (0):\n- keine")
			self["updates"].show()
		except:
			import traceback
			traceback.print_exc()

	def cancel(self):
		self.close()


#=========== for autoupdatecheck ================================

gl_udc_session = None

def updatecheck(udc_session):
	import Screens.Standby
	if Screens.Standby.inStandby:
		print "[UpdateCheck] inStandby - don't check for updates"
		return
	
	global gl_udc_session
	gl_udc_session = udc_session
	
	global container
	global appClosed_conn
	global appErr_conn
	
	global timeoutTimer
	global timeoutTimer_conn

	timeoutTimer = eTimer()
	timeoutTimer_conn = timeoutTimer.timeout.connect(CheckTimeout)
	timeoutTimer.start(int(config.plugins.updatecheck.update_timeout.value)*1000)
	
	container = eConsoleAppContainer()
	appClosed_conn = container.appClosed.connect(startUpdateCheck)
	appErr_conn = container.stderrAvail.connect(consoleError)
	#self.dataAvail_conn = self.container.dataAvail.connect(consoledataAvail)
	container.execute("apt-get update")
	print "[UpdateCheck] start Feed-Update"

def CheckTimeout():
	print "[UpdateCheck] CheckTimeout"
	global timeoutTimer
	global timeoutTimer_conn
	global container
	
	timeoutTimer.stop()
	timeoutTimer = None
	timeoutTimer_conn = None
	
	if container.running():
		print "[UpdateCheck] timeout for eConsoleAppContainer"
		container.kill()
		AddPopup(_("\n = UpdateCheck = \n\n'update' timed out!!!"), MessageBox.TYPE_INFO, int(config.plugins.updatecheck.popups_timeout.value),'UDC_PopUp_Update')

def consoleError(retval):
	print "[UpdateCheck] consoleError", retval

#def consoledataAvail(str):
#	print "[UpdateCheck] consoledataAvail", str

def startUpdateCheck(retval):
	
	global gl_udc_session
	session = gl_udc_session
	
	print "[UpdateCheck] start UpdateCheck" #, retval
	
	global timeoutTimer
	global timeoutTimer_conn
	
	timeoutTimer.stop()
	timeoutTimer = None
	timeoutTimer_conn = None
	
	command = Command("apt list --upgradable")
	command.run(timeout=int(config.plugins.updatecheck.update_timeout.value))
				
	if command.timeout:
		AddPopup(_("\n = UpdateCheck = \n\n'list --upgradable' timed out!!!"), MessageBox.TYPE_INFO, int(config.plugins.updatecheck.popups_timeout.value),'UDC_PopUp_Update')
		return
	elif command.process.returncode != 0:
		AddPopup("\n = UpdateCheck = \n\n'list --upgradable' - "+ str(command.process.returncode) + " - " + str(command.err), MessageBox.TYPE_INFO, int(config.plugins.updatecheck.popups_timeout.value),'UDC_PopUp_Update')
		return
	
	check_update_txt = command.out
	
	updates = check_update_txt.split("\n")
	#print "[UpdateCheck] check_update:", check_update_txt
	#print "[UpdateCheck] update:", updates, len(updates)
	
	UpdateExist = False
	
	hold_packages = []
	
	try:
		if config.plugins.updatecheck.ignore_hold.value:
			#read hold-packages
			hold_packages_txt = ""
			try:
				hold_packages_txt = subprocess.check_output("apt-mark showhold", shell=True)
				#hold_packages_txt = subprocess.check_output("cat /var/lib/dpkg/status | grep -B5 hold | grep Package", shell=True)
			except:
				print "[UpdateCheck] no hold packages found"
				pass

			#prepare hold-list
			if len(hold_packages_txt):
				packages = hold_packages_txt.split("\n")
				for package in packages:
					if package: hold_packages.append(str(package) + "/")
			
			#clean updates-list by hold-list
			for hold_package in hold_packages:
				index = [i for i,x in enumerate(updates) if hold_package in x]
				if len(index):
					#print "[UpdateCheck] index:", index, updates[index[0]]
					del updates[index[0]]
		
	except:
		import traceback
		traceback.print_exc()
	
	if len(updates)==3:
		check_update_txt = _("Es liegt 1 Feed-Update vor!") + "\n - " + str(updates[1].split("/")[0])
		print "[UpdateCheck] 1 update found"
		UpdateExist = True
	elif len(updates)>3:
		check_update_txt = _("Es liegen %s Feed-Updates vor!") % str(len(updates)-2)
		check_update_txt += "\n - " + str(updates[1].split("/")[0]) + "\n - " + str(updates[2].split("/")[0])
		if len(updates)>4:
			check_update_txt += "\n - " + str(updates[3].split("/")[0])
		if len(updates)>5:
			check_update_txt += "\n ..."
		
		print "[UpdateCheck] %s updates found" % str(len(updates)-2)
		UpdateExist = True
	else:
		check_update_txt = _("keine Feed-Updates gefunden!")
		print "[UpdateCheck] no updates found"
	
	if UpdateExist:
		if config.plugins.updatecheck.enable_autocheck.value != "False":
			if config.plugins.updatecheck.show_updatequestion.value is False:
				AddPopup("\n = UpdateCheck = \n\n" + check_update_txt + "\n\n\n" + _("zum Update den SoftwareManager öffnen"), MessageBox.TYPE_INFO, int(config.plugins.updatecheck.popups_timeout.value),'UDC_PopUp_Update')
			else:
				#open only if enigma is start complete
				#from Tools import Notifications
				#Notifications.AddNotification(UpdateInfo, check_update_txt)
				session.open(UpdateInfo, check_update_txt)
	else:
		if config.plugins.updatecheck.enable_autocheck.value == "True":
			AddPopup("\n = UpdateCheck = \n\n" + check_update_txt, MessageBox.TYPE_INFO, int(config.plugins.updatecheck.popups_timeout.value),'GPU_PopUp_Update')
	
	#set lastAutoCheckTime
	config.plugins.updatecheck.lastAutoCheck.value = int(time.time())
	config.plugins.updatecheck.lastAutoCheck.save()

#check for Merlin-Image
global isMerlin 
isMerlin = False
try:
 from Components.Merlin import MerlinImage
 isMerlin = True
except: pass

class UpdateInfo(Screen):
	
	skin = """ <screen position="0,0" size="0,0" title="Update Info" flags="wfNoBorder"></screen>"""

	def __init__(self, session, UpdateText=""):
		self.session = session
		Screen.__init__(self, session)
		
		try:
			self.UpdateText = UpdateText
			self.firsttime = True
			self.onShown.append(self.showmsg)
			self.onLayoutFinish.append( self.LayoutFinish )
			
		except:
			import traceback
			traceback.print_exc()

	def LayoutFinish(self):
		
		self.setTitle("UpdateCheck - Updateinfo")

	def showmsg(self):
		
		if self.firsttime:
			self.firsttime = False
			default_answer = False
			if config.plugins.updatecheck.updatequestion_defaultanswer.value == "True":
				default_answer = True
			self.session.openWithCallback(self.openUDC, MessageBox, "\n = UpdateCheck = \n\n" + self.UpdateText +"\n\n\nSoll der SoftwareManager geöffnet werden?", MessageBox.TYPE_YESNO, timeout = int(config.plugins.updatecheck.popups_timeout.value), default = default_answer)
	
	def openUDC(self, confirm):
		
		if confirm:
			#check to open GP3/GP4 AddonManager
			if fileExists(resolveFilename(SCOPE_PLUGINS, "GP4/geminiaddonmanager/gaddon.pyo")):
				print "[UpdateCheck] import GP4 gAddons"
				from Plugins.GP4.geminiaddonmanager.gaddon import gAddonScreen
				self.session.open(gAddonScreen)
				self.close()
			
			#check to open GP3 AddonManager
			elif fileExists(resolveFilename(SCOPE_PLUGINS, "Bp/geminimain/gAddons.pyo")):
				print "[UpdateCheck] import GP3 gAddons"
				from Plugins.Bp.geminimain.gAddons import gAddons
				self.session.open(gAddons)
				self.close()
			
			#check for Merlin
			elif isMerlin and fileExists(resolveFilename(SCOPE_PLUGINS, "SystemPlugins/SoftwareManager/UpdatePlugin.py")):
				self.session.openWithCallback(self.runUpgrade_Merlin, MessageBox, _("Do you want to update your Dreambox?")+"\n"+_("\nAfter pressing OK, please wait!"))
			
			#check for Newnigma2
			elif fileExists(resolveFilename(SCOPE_PLUGINS, "newnigma2/paketmanager/Paketmanager.pyo")):
				from Plugins.newnigma2.tools.NewNigma2Stuff import __
				message = __('Do you really want to Upgrade?')
				self.session.openWithCallback(self.runUpgrade_Newnigma2, MessageBox, message, type=MessageBox.TYPE_YESNO, timeout=20, default=False)
			
			#check to open DMM-SoftwareManager
			elif fileExists(resolveFilename(SCOPE_PLUGINS, "SystemPlugins/SoftwareManager/plugin.py")):
				from Plugins.SystemPlugins.SoftwareManager.plugin import PluginManager
				self.session.open(PluginManager)
				self.close()
			else:
				self.session.open(MessageBox, _("The Softwaremanagement extension is not installed!\nPlease install it."), type = MessageBox.TYPE_INFO,timeout = 10 )
				self.close()
		else:
			self.close()

	def runUpgrade_Merlin(self, result):
		if result:
			from Plugins.SystemPlugins.SoftwareManager.UpdatePlugin import UpdatePlugin
			self.session.open(UpdatePlugin)
		self.close()

	def runUpgrade_Newnigma2(self, result):
		if result:
			from Plugins.newnigma2.paketmanager.Paketmanager import InstallLog
			self.session.open(InstallLog, 3, None, None, "/usr/lib/enigma2/python/Plugins/newnigma2/paketmanager")
		self.close()
